xquery version "3.1";

 (:~
 : This module generates register as TEI-XML for person entities from the editions-TEI
 : and for artworks from lido-XML
 : @author Florian Barth
 : @version 1.0
 : @since 1.0
 :)


module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation";

declare namespace index="https://mollenhauer-edition.de/index";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace pnd="http://textgrid.info/namespaces/vocabularies/pnd";
declare namespace lido="http://www.lido-schema.org";

declare function kmg-register-transformation:persons-all-references($sade-data-path as xs:string*){
    for $doc in collection($sade-data-path)
    let $persons := $doc//tei:text//tei:name[@type='person']
    return
        for $person in $persons
        return
            (: if $person/@ref is a reference to xml:id replace the hashtag
            and search for xml:id :)
            if (matches($person/@ref, '^#')) then
                    $doc//.[@xml:id=replace($person/@ref, '#', '')]
            else
                $person
};

declare function kmg-register-transformation:persons-unique-references($sade-data-path as xs:string*){
    for $person_ref in distinct-values(kmg-register-transformation:persons-all-references($sade-data-path)/@ref)
    let $stripped_id := tokenize($person_ref, ":")[2]
    return
        try{
            let $data := json-doc("https://lobid.org/gnd/" || $stripped_id || ".json")
            return
                element index:entry {
                    attribute index:ref { $person_ref },
                    element index:name { $data("preferredName") },
                    element index:birth { $data("dateOfBirth")(1) },
                    element index:death { $data("dateOfDeath")(1) }
                }
        } catch * {
            element index:entry {
                attribute index:ref { $person_ref },
                element index:name { "_Wrong GND-id format!" }
            }
        }
};

declare function kmg-register-transformation:music-all-references($sade-data-path as xs:string*){
    for $doc in collection($sade-data-path)
    let $works := $doc//tei:text//tei:name[@type='music']
    return
        for $work in $works
        return
            (: if $person/@ref is a reference to xml:id replace the hashtag
            and search for xml:id :)
            if (matches($work/@ref, '^#')) then
                    $doc//.[@xml:id=replace($work/@ref, '#', '')]
            else
                $work
};

declare function kmg-register-transformation:music-unique-references($sade-data-path as xs:string*){
    for $workRef in distinct-values(kmg-register-transformation:music-all-references($sade-data-path)/@ref)
    let $stripped_id := tokenize($workRef, ":")[2]
    return
        try{
            let $data := json-doc("https://lobid.org/gnd/" || $stripped_id || ".json")
            let $name := $data("preferredName")
            let $composer := $data("firstComposer")(1)("label")
            return
                element index:entry {
                    attribute index:ref { $workRef },
                    element index:name { $name },
                    element index:composer { $composer }
                }
        } catch * {
            element index:entry {
                attribute index:ref { $workRef },
                element index:name { "_Wrong GND-id format!" }
            }
        }
};

declare function kmg-register-transformation:artworks-all-titles($sade-data-path as xs:string*){
    for $doc in collection($sade-data-path)
    let $title := $doc//lido:titleSet/lido:appellationValue[@xml:lang="de"]/text()
    let $tg-id := $doc//lido:lidoRecID/text()
    let $tg-id := if ( contains($tg-id, ".") ) then substring-before($tg-id, ".") else $tg-id
    return
        element index:entry {
            attribute index:ref { $tg-id },
            element index:name { $title }
        }
};

declare function kmg-register-transformation:return-register($sade-data-path as xs:string*, $register-type as xs:string*){
    let $body :=
        if ($register-type = "persons")
        then
            kmg-register-transformation:persons-unique-references($sade-data-path)
        else if ($register-type = "artworks")
        then
            kmg-register-transformation:artworks-all-titles($sade-data-path)
        else if ($register-type = "music")
        then
            kmg-register-transformation:music-unique-references($sade-data-path)
        else
            ()

    return
        element {fn:QName("https://mollenhauer-edition.de/index", "index")} {
            attribute xml:lang { "de" },
            attribute type { $register-type },
            element head {
                element title { "Index of " || $register-type || " entities"},
                element publisher {
                    element name { "Klaus Mollehnauer Gesamtausgabe" }
                }
            },
            element body {
                $body
            }
        }
};
